local _, NAMESPACE = ...
local MAX_ARENA_ENEMIES = NAMESPACE.options.maxArenaFrames
local SetActiveFrame
local icons = {}

if not IsAddOnLoaded("Blizzard_ArenaUI") then
    LoadAddOn("Blizzard_ArenaUI")
end

do
    local GetTime, GetSpellTexture = GetTime, GetSpellTexture

    local function SetIndicators(frame, testMode)
        local applied = not testMode and frame.applied + 1 or random(1, 3)
        frame.applied = applied

        if frame.border then
            frame.border:SetVertexColor(1, applied == 2 and 1 or 0, 0, applied > 1 and 1 or 0)
        end

        if frame.text then
            return frame.text:SetTextColor(1, applied <= 2 and 1 or 0, applied == 1 and 1 or 0, 1)
        end
    end

    function SetActiveFrame(frame, spellID, testMode)
        if frame then
            if not frame.hasTexture then
                frame.icon:SetTexture(GetSpellTexture(spellID))
            end

            frame.time = GetTime()
            frame:SetCooldown(frame.time, not testMode and 18 or random(6, 18))
            return SetIndicators(frame, testMode)
        end
    end
end

-- Setup frames
do
    local _, englishClass = UnitClass("player")
    local categories = NAMESPACE.categories
    local options = NAMESPACE.options

    local ICON_SIZE = options.iconSize
    local REVERSE_GROW = options.iconReverse
    local DEFAULT_POS_X = not REVERSE_GROW and -24 or 100
    local DEFAULT_POS_Y = 22

    local function FrameOnShow(self)
        local posX = DEFAULT_POS_X
        local oppNumber = self.oppNumber

        for i = 1, #categories do
            if icons[categories[i]..oppNumber].shown then
                posX = not REVERSE_GROW and posX - ICON_SIZE or posX + ICON_SIZE
            end
        end

        self:SetPoint("LEFT", posX, DEFAULT_POS_Y)
        self.shown = true
    end

    local function FrameOnHide(self)
        local posX = DEFAULT_POS_X
        local oppNumber = self.oppNumber
        local time = self.time
        self.shown = false
        self.applied = 0

        for i = 1, #categories do
            local frame = icons[categories[i]..oppNumber]

            if frame.shown then
                if frame.time > time or frame:GetCooldownTimes() > self:GetCooldownTimes() then
                    frame:SetPoint("LEFT", posX, DEFAULT_POS_Y)
                    posX = not REVERSE_GROW and posX - ICON_SIZE or posX + ICON_SIZE
                end
            end
        end

        if self:GetCooldownTimes() > 0 then
            return self:SetCooldown(0, 0) -- Stop timers when hidden by parent frame (leaving arena etc)
        end
    end

    for i = 1, MAX_ARENA_ENEMIES do
        local arenaFrame = _G["ArenaEnemyFrame"..i]

        for j = 1, #categories do
            local frame = CreateFrame("Cooldown", nil, arenaFrame, "CooldownFrameTemplate")
            frame.time, frame.applied, frame.oppNumber, frame.shown = 0, 0, i, false
            frame:ClearAllPoints()
            frame:SetHideCountdownNumbers(options.timerTextOff)
            frame:SetSize(ICON_SIZE, ICON_SIZE)
            frame:SetDrawSwipe(options.timerSpiral)
            frame:SetDrawEdge(false)
            frame:EnableMouse(false)
            frame:SetFrameLevel(2)
            frame:SetSwipeColor(0, 0, 0, 0.65)
            frame:Hide()

            frame.icon = frame:CreateTexture(nil, "BACKGROUND")
            frame.icon:SetAllPoints(frame)
            frame.icon:SetTexture(options.textures[englishClass] and GetSpellTexture(options.textures[englishClass][categories[j]]))
            frame.hasTexture = frame.icon:GetTexture() and true or false

            if options.borderColors then
                frame.border = frame:CreateTexture(nil, "OVERLAY")
                frame.border:SetPoint("TOPLEFT", -1, 1)
                frame.border:SetPoint("BOTTOMRIGHT", 1, -1)
                frame.border:SetTexture("Interface\\BUTTONS\\UI-Quickslot-Depress")
            end

            local cdText = frame:GetRegions()
            cdText:SetFont(cdText:GetFont(), options.timerTextSize)
            frame.text = not options.timerTextOff and options.timerColors and cdText or false

            frame:SetScript("OnShow", FrameOnShow)
            frame:SetScript("OnHide", FrameOnHide)
            
            --icons["Stun" .. oppNumber]
            icons[categories[j]..i] = frame
        end
    end

    NAMESPACE.options = nil
end

do
    local spellIDs = NAMESPACE.spellList
    local events = CreateFrame("Frame")
    local arenaGUIDs = {}

    do
        local UnitGUID = UnitGUID

        local function GetOpponentNumber(self, destGUID)
            if not destGUID then return end
            local oppNumber = 0

            for i = 1, MAX_ARENA_ENEMIES do
                if UnitGUID("arena"..i) == destGUID then
                    oppNumber = i
                    break
                end
            end

            self[destGUID] = oppNumber -- note: 0 will be treated as a blacklist
            return oppNumber
        end

        -- "memoization" for guids
        setmetatable(arenaGUIDs, { __index = GetOpponentNumber })
    end

    local function EventHandler(self, event, _, eventType, _, _, _, _, _, destGUID, _, _, _, spellID, _, _, auraType)
        if event == "COMBAT_LOG_EVENT_UNFILTERED" then
            if not auraType or auraType ~= "DEBUFF" then return end

            if eventType == "SPELL_AURA_REMOVED" or eventType == "SPELL_AURA_REFRESH" then
                local category = spellIDs[spellID]
                if category then
                    local oppNumber = arenaGUIDs[destGUID]
                    if oppNumber and oppNumber > 0 then -- found & not blacklisted
                        return SetActiveFrame(icons[category..oppNumber], spellID)
                    end
                end
            end
        else -- Player entering world
            if select(2, IsInInstance()) == "arena" then
                wipe(arenaGUIDs)
                self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
            else
                self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
            end
        end
    end

    events:RegisterEvent("PLAYER_ENTERING_WORLD")
    events:SetScript("OnEvent", EventHandler)
    events:Hide()
end

SLASH_DIMINISH1 = "/diminish"
SlashCmdList["DIMINISH"] = function()
    if InCombatLockdown() or UnitAffectingCombat("player") then
        return print("Must leave combat")
    end

    local showFlag = not ArenaEnemyFrames:IsShown()
    local categories = NAMESPACE.categories
    ArenaEnemyFrames:SetShown(showFlag)
    for i = 1, MAX_ARENA_ENEMIES do
        _G["ArenaEnemyFrame"..i]:SetShown(showFlag)
        
        if showFlag then
            for j = 1, #categories do
                SetActiveFrame(icons[categories[j]..i], 118, true)
            end
        end
    end
end
